@@ -1,31 +1,38 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (c) 1998-2006 Michal Trojnara <Michal.Trojnara@mirt.net>
- *                 All Rights Reserved
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   Copyright (C) 1998-2008 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
+ *   This program is free software; you can redistribute it and/or modify it
+ *   under the terms of the GNU General Public License as published by the
+ *   Free Software Foundation; either version 2 of the License, or (at your
+ *   option) any later version.
+ * 
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- *   In addition, as a special exception, Michal Trojnara gives
- *   permission to link the code of this program with the OpenSSL
- *   library (or with modified versions of OpenSSL that use the same
- *   license as OpenSSL), and distribute linked combinations including
- *   the two.  You must obey the GNU General Public License in all
- *   respects for all of the code used other than OpenSSL.  If you modify
- *   this file, you may extend this exception to your version of the
- *   file, but you are not obligated to do so.  If you do not wish to
- *   do so, delete this exception statement from your version.
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ *   See the GNU General Public License for more details.
+ * 
+ *   You should have received a copy of the GNU General Public License along
+ *   with this program; if not, see <http://www.gnu.org/licenses>.
+ * 
+ *   Linking stunnel statically or dynamically with other modules is making
+ *   a combined work based on stunnel. Thus, the terms and conditions of
+ *   the GNU General Public License cover the whole combination.
+ * 
+ *   In addition, as a special exception, the copyright holder of stunnel
+ *   gives you permission to combine stunnel with free software programs or
+ *   libraries that are released under the GNU LGPL and with code included
+ *   in the standard release of OpenSSL under the OpenSSL License (or
+ *   modified versions of such code, with unchanged license). You may copy
+ *   and distribute such a system following the terms of the GNU GPL for
+ *   stunnel and the licenses of the other code concerned.
+ * 
+ *   Note that people who make modified versions of stunnel are not obligated
+ *   to grant this special exception for their modified versions; it is their
+ *   choice whether to do so. The GNU General Public License gives permission
+ *   to release a modified version without this exception; this exception
+ *   also makes it possible to release a modified version which carries
+ *   forward this exception.
  */
 
 #include "common.h"
@@ -109,7 +116,7 @@ static void scan_waiting_queue(void) {
     short *signal_revents;
     static int max_nfds=0;
     static struct pollfd *ufds=NULL;
-    
+
     time(&now);
     /* count file descriptors */
     min_timeout=-1;
@@ -125,7 +132,7 @@ static void scan_waiting_queue(void) {
         ufds=realloc(ufds, nfds*sizeof(struct pollfd));
         if(!ufds) {
             s_log(LOG_CRIT, "Memory allocation failed");
-            exit(1);
+            die(1);
         }
         max_nfds=nfds;
     }
@@ -230,6 +237,7 @@ int s_poll_wait(s_poll_set *fds, int sec, int msec) {
             s_log(LOG_DEBUG, "Current context: %ld", ready_head->id);
             if(to_free) {
                 s_log(LOG_DEBUG, "Releasing context %ld", to_free->id);
+                free(to_free->stack);
                 free(to_free);
                 to_free=NULL;
             }
@@ -239,6 +247,7 @@ int s_poll_wait(s_poll_set *fds, int sec, int msec) {
         /* it's illegal to deallocate the stack of the current context */
         if(to_free) {
             s_log(LOG_DEBUG, "Releasing context %ld", to_free->id);
+            free(to_free->stack);
             free(to_free);
         }
         to_free=ctx;
@@ -383,12 +392,12 @@ int signal_pipe_init(void) {
         closesocket(pipe_in);
     } else {
         sockerror("select");
-        exit(1);
+        die(1);
     }
 #else /* __INNOTEK_LIBC__ */
     if(pipe(signal_pipe)) {
         ioerror("pipe");
-        exit(1);
+        die(1);
     }
     alloc_fd(signal_pipe[0]);
     alloc_fd(signal_pipe[1]);
